#include <PID_v1.h>
#include "soc/timer_group_struct.h"
#include "soc/timer_group_reg.h"


// PID - Regler
double M1Setpoint, M1Input, M1Output;
double M2Setpoint, M2Input, M2Output;
double KP = 0.75, KI = 2.5, KD = 0.005; //KP = 0.03, KI = 38, KD = 0; // 

PID M1_PID(&M1Input, &M1Output, &M1Setpoint, KP, KI, KD, DIRECT); 
PID M2_PID(&M2Input, &M2Output, &M2Setpoint, KP, KI, KD, DIRECT);

// TaskHandle
TaskHandle_t RegelTask, FeedbackTask;

//ESP-WROOM-32
#define D23 23
#define D22 22
#define D21 21
#define D19 19
#define D18 18
#define D5 5
#define D4 4

#define IN1 D22
#define IN2 D21
#define IN3 D19
#define IN4 D18
#define INTM1_PIN D4
#define INTM2_PIN D23
#define INH D5

//PWM Channels
int PWMIN1 = 0;
int PWMIN2 = 2;
int PWMIN3 = 4;
int PWMIN4 = 6;

int M1EncoderCNTR = 0;
//int M1RotationCNTR = 0;
int M2EncoderCNTR = 0;
//int M2RotationCNTR = 0;
//int rotations = 0;

int MAXSPEED = 4095;
int FREQUENCY = 490;

int i = 0; //Rampincrement

//float Drehzahl = 0;
float SkalierungM1 = 1;
float SkalierungM2 = 1;

//boolean reverse = 0;

boolean reverse1 = 0;
boolean reverse2 = 0;

boolean Anfahrtsrampe = 0;

//boolean _flipM1 = false;
//boolean _flipM2 = false;

//int sprungantw[3000] = {NULL};

int turnSpeed = 0;

// Delay in der Main Loop
unsigned long mSec = 10000;
unsigned long currMillis = 0, prevMillis = millis();

unsigned long previousMillis = 0;
const long Interval = 250;
unsigned long currentMillis = 0; 

double M1Umdrehungen = 0.0;
double M2Umdrehungen = 0.0;

// INTERRUPTS
void M1handleInterrupt() {
    M1EncoderCNTR++;    
}


void M2handleInterrupt() {
    M2EncoderCNTR++;    
}


void setMotorScale(int turnSpeedM1, int turnSpeedM2) {
  init(); // initialize if necessary
  reverse1 = 0;
  reverse2 = 0;

  if (turnSpeedM1 < 0)
  {
    turnSpeedM1 = -turnSpeedM1; // make speed a positive quantity
    reverse1 = 1;    // preserve the direction
  }
  if (turnSpeedM2 < 0)
  {
    turnSpeedM2 = -turnSpeedM2; // make speed a positive quantity
    reverse2 = 1;    // preserve the direction
  }
  if (turnSpeedM1 > MAXSPEED)  // max PWM duty cycle
    turnSpeedM1 = MAXSPEED;
  if (turnSpeedM2 > MAXSPEED)
    turnSpeedM2 = MAXSPEED;

  if (turnSpeedM1 == turnSpeedM2) {
    SkalierungM1 = 1;
    SkalierungM2 = 1;
    turnSpeed = turnSpeedM2;
  }
  else {
    if (turnSpeedM2 == 0) {
      SkalierungM1 = 1;
      SkalierungM2 = 0;
      turnSpeed = turnSpeedM1;
    }
    else {
      SkalierungM1 = turnSpeedM1/turnSpeedM2;
      SkalierungM2 = 1;
      turnSpeed = turnSpeedM2;   
    }  
  }
}


//Set both Motors on the same value
/*
Motorgeschwindigkeit sollte nur noch über setMotorSpeeds eingestellt werden!!!
turnSpeed: Im Rampmode irrelevant, kann 0 gesetzt werden
turnSpeedM1: Motorspeed Motor 1 (-MAXSPEED bis MAXSPEED) -MAXSPEED ist vorwärts (Bereich von -800 bis 800 sollte vermieden werden, da sich hier der Motor nicht bewegt)
turnSpeedM2: Motorspeed Motor 2 (-MAXSPEED bis MAXSPEED) MAXSPEED ist vorwärts (Bereich von -800 bis 800 sollte vermieden werden, da sich hier der Motor nicht bewegt)
Ramp: Auf 1 setzen für Anfahrtsrampe
Bsp: setMotorSpeeds(0, -MAXSPEED, MAXSPEED/2, 1);  //Vorwärtslauf des Roboters mit unterschiedlichen Geschwindigkeiten der Räder
*/
void setMotorSpeeds(int turnSpeed, int turnSpeedM1, int turnSpeedM2, boolean Ramp)  {
  if (Ramp) {
    Anfahrtsrampe = true;
    setMotorScale(turnSpeedM1, turnSpeedM2);
  }
  else {
    Anfahrtsrampe = false;
    setMotorScale(turnSpeedM1, turnSpeedM2);  
  }
}


void RegelTaskcode( void * parameter) {
  while(1) {
    TIMERG0.wdt_wprotect=TIMG_WDT_WKEY_VALUE;
    TIMERG0.wdt_feed=1;
    TIMERG0.wdt_wprotect=0;

    if (reverse1 == 1 && reverse2 == 1) {
      // Clockwise
      digitalWrite(INH, LOW);
      ledcWrite(PWMIN2, 0);
      ledcWrite(PWMIN4, 0);
      if (Anfahrtsrampe) {
        if (i < turnSpeed) {
          for (i; i < turnSpeed; i++) { 
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;
            
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2);   
                      
              ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN3, MAXSPEED-M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
          }   
        }
        if (i > turnSpeed) {
          for (i; i > turnSpeed; i--) { 
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;
            
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();     
              delay(2);  
                      
              ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN3, MAXSPEED-M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
          }     
        }
        if (i == turnSpeed) {        
          M1Setpoint = i*SkalierungM1;
          M2Setpoint = i*SkalierungM2;
          
          if (i >= 800) {
             M1Input = 2380 * M1Umdrehungen; //2380
            M1_PID.Compute();      
            delay(2);       
            M2Input = 2380 * M2Umdrehungen; //2380
            M2_PID.Compute();     
            delay(2); 
                   
            ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
            ledcWrite(PWMIN3, MAXSPEED-M2Output);
            delay(3);
            digitalWrite(INH, HIGH);
          }
        }
      }
      else {
        i = turnSpeed;
        M1Setpoint = i*SkalierungM1;
        M2Setpoint = i*SkalierungM2;
       
        if (i >= 800) {
          M1Input = 2380 * M1Umdrehungen; //2380
          M1_PID.Compute();      
          delay(2);       
          M2Input = 2380 * M2Umdrehungen; //2380
          M2_PID.Compute();     
          delay(2);        
          ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
          ledcWrite(PWMIN3, MAXSPEED-M2Output);
          delay(3);
          digitalWrite(INH, HIGH);
        }
      }
    }
    if (reverse1 == 0 && reverse2 == 0) {
      digitalWrite(INH, LOW);
      ledcWrite(PWMIN1, 4095);
      ledcWrite(PWMIN3, 4095);
      if (Anfahrtsrampe) {
        if (i < turnSpeed) {
          for (i; i < turnSpeed; i++) {          
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;
            
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2); 
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2);     
                   
              ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN4, M2Output); // use analogWrite from 0 to 255
              delay(3);
              digitalWrite(INH, HIGH);
            }
          } 
        }
        if (i > turnSpeed) {
          for (i; i > turnSpeed; i--) {     
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;
                        
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();
              delay(2);
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();
              delay(2);  
                      
              ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN4, M2Output); // use analogWrite from 0 to 255
              delay(3);
              digitalWrite(INH, HIGH);
            }
          } 
        }
        if (i == turnSpeed) {        
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;    
            
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();
              delay(2);
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();
              delay(2);
              
              ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN4, M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
        }
      }
      else {
        i = turnSpeed;
        M1Setpoint = i*SkalierungM1;
        M2Setpoint = i*SkalierungM2;
        
        if (i >= 800) {
          M1Input = 2380 * M1Umdrehungen; //2380
          M1_PID.Compute();
          delay(2);
          M2Input = 2380 * M2Umdrehungen; //2380
          M2_PID.Compute();
          delay(2);
          
          ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
          ledcWrite(PWMIN4, M2Output);
          delay(3);
          digitalWrite(INH, HIGH);
        }
      }
    }
    if (reverse1 == 1 && reverse2 == 0) {
      digitalWrite(INH, LOW);
      ledcWrite(PWMIN2, 0);
      ledcWrite(PWMIN3, 4095);
      if (Anfahrtsrampe) {
        if (i < turnSpeed) {
          for (i; i < turnSpeed; i++) {               
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;  
            
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2);   
                     
              ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN4, M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
          }  
        }
        if (i > turnSpeed) {
          for (i; i > turnSpeed; i--) {               
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;   
            
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2); 
                       
              ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN4, M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
          }  
        }
        if (i == turnSpeed) {        
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2; 
           
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2);          
              ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN4, M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
        }
      }
      else {
        i = turnSpeed;
        M1Setpoint = i*SkalierungM1;
        M2Setpoint = i*SkalierungM2; 
        
        if (i >= 800) {
          M1Input = 2380 * M1Umdrehungen; //2380
          M1_PID.Compute();      
          delay(2);        
          M2Input = 2380 * M2Umdrehungen; //2380
          M2_PID.Compute();      
          delay(2);          
          ledcWrite(PWMIN1, MAXSPEED-M1Output); // use analogWrite from 0 to 255
          ledcWrite(PWMIN4, M2Output);
          delay(3);
          digitalWrite(INH, HIGH);
        }        
      }
    }
    if (reverse1 == 0 && reverse2 == 1) {
      digitalWrite(INH, LOW);
      ledcWrite(PWMIN1, 4095);
      ledcWrite(PWMIN4, 0);
      if (Anfahrtsrampe) {
        if (i < turnSpeed) {
          for (i; i < turnSpeed; i++) {  
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;  
            
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2);      
                  
              ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN3, MAXSPEED-M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
          }  
        }
        if (i > turnSpeed) {
          for (i; i > turnSpeed; i--) { 
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;
              
            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2);     
                   
              ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN3, MAXSPEED-M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
          }  
        }
        if (i == turnSpeed) {            
            M1Setpoint = i*SkalierungM1;
            M2Setpoint = i*SkalierungM2;  

            if (i >= 800) {
              M1Input = 2380 * M1Umdrehungen; //2380
              M1_PID.Compute();      
              delay(2);        
              M2Input = 2380 * M2Umdrehungen; //2380
              M2_PID.Compute();      
              delay(2);          
              ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
              ledcWrite(PWMIN3, MAXSPEED-M2Output);
              delay(3);
              digitalWrite(INH, HIGH);
            }
        }
      }
      else {
        i = turnSpeed;
        M1Setpoint = i*SkalierungM1;
        M2Setpoint = i*SkalierungM2;  

        if (i >= 800) {
          M1Input = 2380 * M1Umdrehungen; //2380
          M1_PID.Compute();      
          delay(2);        
          M2Input = 2380 * M2Umdrehungen; //2380
          M2_PID.Compute();      
          delay(2);          
          ledcWrite(PWMIN2, M1Output); // use analogWrite from 0 to 255
          ledcWrite(PWMIN3, MAXSPEED-M2Output);
          delay(3);
          digitalWrite(INH, HIGH);
        }
      }
    }    
  vTaskDelay(50);
  } 
}


void FeedbackTaskcode ( void * parameter) {
  while(1) {
    TIMERG0.wdt_wprotect=TIMG_WDT_WKEY_VALUE;
    TIMERG0.wdt_feed=1;
    TIMERG0.wdt_wprotect=0;
    
    currentMillis = millis();

    if (currentMillis - previousMillis >= Interval) {
      previousMillis = currentMillis;
      M1Umdrehungen = 4 * (double)M1EncoderCNTR / (127.8 * 2);    // = Umdrehungen pro Sekunde; *2 wegen Übersetzung zu Rad
      M1EncoderCNTR = 0;
      M2Umdrehungen = 4 * (double)M2EncoderCNTR / (127.8 * 2);    // = Umdrehungen pro Sekunde; *2 wegen Übersetzung zu Rad
      M2EncoderCNTR = 0;
    }
    vTaskDelay(10);
  }
}


void setup() {
  // put your setup code here, to run once:
  Serial.begin(115200);

  xTaskCreatePinnedToCore(
      RegelTaskcode, /* Function to implement the task */
      "RegelTask", /* Name of the task */
      10000,  /* Stack size in words */
      NULL,  /* Task input parameter */
      0,  /* Priority of the task */
      &RegelTask,  /* Task handle. */
      0); /* Core where the task should run */

   xTaskCreatePinnedToCore(
      FeedbackTaskcode, /* Function to implement the task */
      "FeedbackTask", /* Name of the task */
      10000,  /* Stack size in words */
      NULL,  /* Task input parameter */
      0,  /* Priority of the task */
      &FeedbackTask,  /* Task handle. */
      0); /* Core where the task should run */

  //initialize the variables we're linked to
  M1Input = 0;
  M1Setpoint = 0;
  
  M2Input = 0;
  M2Setpoint = 0;

  //turn the PID on
  M1_PID.SetSampleTime(1/490);
  M1_PID.SetOutputLimits(0,4095);
  M1_PID.SetMode(AUTOMATIC);

  M2_PID.SetSampleTime(1/490);
  M2_PID.SetOutputLimits(0,4095);
  M2_PID.SetMode(AUTOMATIC);

  //Setup Motortreiber
  pinMode(INTM1_PIN, INPUT);  //attach Encodersignal to INTM1_PIN
  pinMode(INTM2_PIN, INPUT);  //attach Encodersignal to INTM2_PIN
  pinMode(IN1, OUTPUT);
  pinMode(IN2, OUTPUT);
  pinMode(IN3, OUTPUT);
  pinMode(IN4, OUTPUT);
  pinMode(INH, OUTPUT);

  digitalWrite(INH, LOW);
  digitalWrite(IN1, LOW);
  digitalWrite(IN2, LOW);
  digitalWrite(IN3, LOW);
  digitalWrite(IN4, LOW);
  
  ledcAttachPin(IN1, PWMIN1);
  ledcSetup(PWMIN1, FREQUENCY, 12);
  
  ledcAttachPin(IN2, PWMIN2);
  ledcSetup(PWMIN2, FREQUENCY, 12);

  ledcAttachPin(IN3, PWMIN3);
  ledcSetup(PWMIN3, FREQUENCY, 12);
  
  ledcAttachPin(IN4, PWMIN4);
  ledcSetup(PWMIN4, FREQUENCY, 12);

  //Setup Interupts for Encodersignal
  attachInterrupt(digitalPinToInterrupt(INTM1_PIN), M1handleInterrupt, CHANGE);
  attachInterrupt(digitalPinToInterrupt(INTM2_PIN), M2handleInterrupt, CHANGE);

  setMotorSpeeds(0, 0, 0, 0);
}


void printMotors() {
  Serial.print("I ");
  Serial.print(i);
  Serial.print(" turnSpeed ");
  Serial.print(turnSpeed);
  Serial.print(" M1Umdrehungen " );
  Serial.print(M1Umdrehungen);
  Serial.print(" M1Output " );
  Serial.print(M1Output);
  Serial.print(" M1Input ");
  Serial.print(M1Input);
  Serial.print(" M1Setpoint ");
  Serial.print(M1Setpoint);

  Serial.print(" M2Umdrehungen " );
  Serial.print(M2Umdrehungen);
  Serial.print(" M2Output ");
  Serial.print(M2Output);
  Serial.print(" M2Input ");
  Serial.print(M2Input);
  Serial.print(" M2Setpoint ");
  Serial.println(M2Setpoint);
}


void loop() {
  /* Haus des Nikolaus
  setMotorSpeeds(0, 1500, 1500, 1); 
  while( currMillis - prevMillis <= 5000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 800, -800, 1); 
  while( currMillis - prevMillis <= 1300) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  /////////////////////////////////////////////////////////////////G2
  setMotorSpeeds(0, 1500, 1500, 1); 
  while( currMillis - prevMillis <= 5000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 800, -800, 1); 
  while( currMillis - prevMillis <= 1300) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  //////////////////////////////////////////////////////G3
  setMotorSpeeds(0, 1500, 1500, 1); 
  while( currMillis - prevMillis <= 5000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 800, -800, 1); 
  while( currMillis - prevMillis <= 1300) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  //////////////////////////////////////////////////////G4
  setMotorSpeeds(0, 1500, 1500, 1); 
  while( currMillis - prevMillis <= 5000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 800, -800, 1); 
  while( currMillis - prevMillis <= 2250) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  ///////////////////////////////////////4Eck zu Ende
  setMotorSpeeds(0, 1500, 1500, 1); 
  while( currMillis - prevMillis <= 6500) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, -800, 800, 1); 
  while( currMillis - prevMillis <= 1300) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  //////////////SPITZE
  setMotorSpeeds(0, 1200, 1200, 1); 
  while( currMillis - prevMillis <= 3435) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, -800, 800, 1); 
  while( currMillis - prevMillis <= 1400) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 1200, 1200, 1); 
  while( currMillis - prevMillis <= 3435) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  /////////letzte Diagonale
  prevMillis = currMillis;
  setMotorSpeeds(0, -800, 800, 1); 
  while( currMillis - prevMillis <= 1400) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 1000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 1500, 1500, 1); 
  while( currMillis - prevMillis <= 7070) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 10000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  //DAS WAR DAS AHUS VOM NIKOLAUS
  */
  // Geradeauslauf
  setMotorSpeeds(0, 1500, 1500, 1); 
  while( currMillis - prevMillis <= 10000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
  setMotorSpeeds(0, 0, 0, 0); 
  while( currMillis - prevMillis <= 5000) {
    currMillis = millis();
    printMotors();
  }
  prevMillis = currMillis;
}
